/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

// last modified 01/20/98

package EDU.rutgers.CORBA.GIOP.messages;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;

/**
 * [CORBA 2.0 Specification 12-20]
 *
 * @see GIOPMessage
 *
 * @see GIOPRequestMessage
 * @see GIOPReplyMessage
 * @see GIOPLocateRequestMessage
 * @see GIOPLocateReplyMessage
 * @see GIOPCloseConnectionMessage
 * @see GIOPMessageErrorMessage
 *
 * @since ACT 0.1
 */
public final class GIOPCancelRequestMessage extends GIOPMessage
{
    private int cancelRequestHeaderSize_; // network size

    /**
     * Empty constructor, it could only be used by the GIOPMessageFactory
     *
     * @since ACT 0.1
     */
    protected GIOPCancelRequestMessage() {
        super(MessageType.CancelRequest);
    }

    /**
     * @param int request_id 
     *
     * @since ACT 0.1
     */
    public GIOPCancelRequestMessage(int request_id) {
        super(MessageType.CancelRequest);

        request_id_ = request_id;

        calCancelRequestHeaderSize();
    }
   
    /**
     * Calculate the header network size. In case the user just resuse this object
     * by replacing some member fields, this method forces to reset the GIOP header
     * message_size field.
     *
     * @since ACT 0.1
     */
    public void calCancelRequestHeaderSize() {
        int twoHeadersSize = super.giopHeaderSize_;

        twoHeadersSize += super.ts_.calAlignGap(twoHeadersSize, 4) + 4; // request_id
        cancelRequestHeaderSize_ = twoHeadersSize - super.giopHeaderSize_;
        super.setMessageSize(cancelRequestHeaderSize_);
    }

    /**
     * Read from the TypeReader stream. 
     * The TypeReader should have the same transfer syntax as the GIOPMessage.ts_ member variable.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see GIOPMessage#ts_
     * @see EDU.rutgers.act.presentation.TypeReader
     *
     * @since ACT 0.1
     */
    public void read(TypeReader in)
		throws java.io.IOException {
        super.read(in);

        request_id_ = in.readLong();
	}

    /**
     * Write the TypeWriter stream. 
     * The TypeWriter should have the same transfer syntax as the GIOPMessage.ts_ member variable.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see GIOPMessage#ts_
     * @see EDU.rutgers.act.presentation.TypeWriter
     *
     * @since ACT 0.1
     */
	public void write(TypeWriter out)
		throws java.io.IOException {
        super.write(out);

		out.writeLong(request_id_);
	}

// Object
/*
    public boolean equals(Object o) {
		if(super.equals(o) && (o != null) && (o instanceof GIOPRequestMessage)) {
            if (o == this) {
                return true;
            }
            GIOPCancelRequestMessage msg = (GIOPCancelRequestMessage)o;
            if (request_id_ == msg.request_id_) {
                return true;
            }
        }
        return false;
    }
*/

    /**
     * Returns a string representation. 
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
        StringBuffer buf = new StringBuffer(super.toString()).
            append("\nGIOP:CancelRequestMessage\n\t").
            append("[Header] ").
            append(" request_id: ").append(request_id_).
            append("[Body] ").append("None");
        return String.valueOf(buf);
    }    
}
