/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT 0.01
 *
 */

package EDU.rutgers.CORBA.presentation;

// JDK
import java.io.Writer;
import java.io.IOException;

// ACT
import ORG.omg.CORBA.CompletionStatus;
import ORG.omg.CORBA.DATA_CONVERSION;

import EDU.rutgers.act.presentation.TypeWriter;
import EDU.rutgers.act.presentation.TransferSyntax;

/**
 * @since ACT 0.01
 */
public class CDRTypeWriter extends TypeWriter
{
    /**
     * The CDR transfer syntax used
     * @since ACT 0.01
     */
    private static TransferSyntax ts_ = new CDRTransferSyntax();

    /**
     * @param Writer actual writer to perform writing
     *
     * @since ACT 0.01
     */
    public CDRTypeWriter(Writer out) {
		super(out);
    }

    /**
     * @param int CDR alignemnt needed
     *
     * @since ACT 0.01
     */
 	protected void setAlignment(int alignment)
		throws IOException {
        int gap = ts_.calAlignGap(index_, alignment); 
		if (gap == 0) {
			return;
		}
		index_ += gap;
	        // CRIS!!!
		// -> replace this with line below: for (; --gap != 0;) {
	        for (; gap-- != 0;) {
			out.write((byte)0);
		}
	}

    /**
     * @since ACT 0.01
     */
	public void writeBoolean(boolean b)
		throws IOException {
        writeOctet((b == false) ? (byte)0 : (byte)1); 
	}

    /**
     * @since ACT 0.01
     */
	public void writeChar(char c)
        throws IOException, DATA_CONVERSION {
        if ((short)c > 0xFF) {
            throw new DATA_CONVERSION(7, CompletionStatus.COMPLETED_NO); 
        }
		writeOctet((byte)c);
	}

    /**
     * @since ACT 0.01
     */
	public void writeOctet(byte b)
		throws IOException {
		// setAlignment(1); <-- could be ommited
		out.write((int)b & 0XFF);
		index_ ++;
	}

    /**
     * @since ACT 0.01
     */
	public void writeShort(short s)
		throws IOException {
		setAlignment(2);
		byte b1 = (byte)(s & 0xFF);
		byte b2 = (byte)((s >> 8) & 0xFF);
        if (littleEndian_) {
			writeOctet(b1);
			writeOctet(b2);
            return; 
        }
		writeOctet(b2);
		writeOctet(b1);
	}

    /**
     * @since ACT 0.01
     */
    public void writeUShort(short us)
		throws IOException {
		writeShort((short)us);
	}

    /**
     * @since ACT 0.01
     */
	public void writeLong(int l)
		throws IOException {
		setAlignment(4);
		byte b1 = (byte)((l >>> 24) & 0xFF);
		byte b2 = (byte)((l >>> 16) & 0xFF);
		byte b3 = (byte)((l >>>  8) & 0xFF);
		byte b4 = (byte)(l & 0xFF);

	// CRIS!!! replace line below:
        // if (littleEndian_) {
        if (!littleEndian_) {
			writeOctet(b1);
			writeOctet(b2);
			writeOctet(b3);
			writeOctet(b4);
			return;
		}
		writeOctet(b4);
		writeOctet(b3);
		writeOctet(b2);
		writeOctet(b1);
	}

    /**
     * @since ACT 0.01
     */
    public void writeULong(int ul)
		throws IOException {
		writeLong((int)ul);
	}

    /**
     * @since ACT 0.01
     */
	public void writeLongLong(long ll)
		throws IOException {
		setAlignment(8);
		int l1 = (int)(ll & 0xFFFFFFFF);
		int l2 = (int)(ll >>> 32);

		if (littleEndian_) {
			writeLong(l1);
			writeLong(l2);
		}
		writeLong(l2);
		writeLong(l1);
	}

    /**
     * @since ACT 0.01
     */
    public void writeULongLong(long ull)
		throws IOException {
		writeLongLong((long)ull);
	}

    /**
     * @since ACT 0.01
     */
	public void writeFloat(float f)
		throws IOException {
		writeLong(Float.floatToIntBits(f));
	}

    /**
     * @since ACT 0.01
     */
    public void writeDouble(double d)
		throws IOException {
		writeLongLong(Double.doubleToLongBits(d));
	}

    /**
     * @since ACT 0.01
     */
	public void writeString(String str)
		throws IOException {
		int len = str.length();
		writeLong(len + 1);
		for (int i = 0 ; i < len ; i++) {
			writeOctet((byte)str.charAt(i));
		}		
		// CRIS!!!
		writeOctet((byte)0);
	}
}
