/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package ORG.omg.CORBA;

/**
 * SystemException status
 * <p>[CORBA 2.0 Specification 3-35]
 *
 * @see SystemException
 *
 * @since ACT 0.1
 */
public final class CompletionStatus
{
    /**
     * COMPLETED_YES type value
     *
     * @since ACT 0.1
     */
    public static final int _COMPLETED_YES = 0;

    /**
     * COMPLETED_NO type value
     *
     * @since ACT 0.1
     */
    public static final int _COMPLETED_NO = 1;

    /**
     * COMPLETED_MAYBE type value
     *
     * @since ACT 0.1
     */
    public static final int _COMPLETED_MAYBE = 2;

    /**
     * COMPLETED_YES type
     * <p>The object implementation has completed processing prior to the exception being raised.
     *
     * @since ACT 0.1
     */
    public static final CompletionStatus COMPLETED_YES = 
		new CompletionStatus(_COMPLETED_YES);

    /**
     * COMPLETED_NO type
     * <p>The object implementation was never initiated prior to the exception being raised.
     *
     * @since ACT 0.1
     */
    public static final CompletionStatus COMPLETED_NO = 
		new CompletionStatus(_COMPLETED_NO); 

    /**
     * COMPLETED_YES type
     * <p>The status of implementation is indertermined
     *
     * @since ACT 0.1
     */
    public static final CompletionStatus COMPLETED_MAYBE = 
		new CompletionStatus(_COMPLETED_MAYBE);

    /**
     * @since ACT 0.1
     */
    private CompletionStatus(int value) {        
        value = value; 
    }

    /**
     * Type value
     *
     * @since ACT 0.1
     */
    private int value;

    /**
     * Retrieve the type value
     *
     * @return int the type value
     *
     * @since ACT %Version%
     */
    public int value() {
        return value; 
    }

    /**
     * From the type integer value, retrieve the CompletionStatus
     *
     * @param int the type integer value
     *
     * @return CompletionStatus the corresponding MessageType
     *
     * @exception BAD_PARAM if the integer value is out of range
     *
     * @since ACT 0.1
     */
    public static final CompletionStatus type(int value) 
		throws BAD_PARAM {
        switch(value) {
            default:
				throw new BAD_PARAM("Enum out of range[0...2]: " + value);

            case _COMPLETED_YES :
                return CompletionStatus.COMPLETED_YES; 

            case _COMPLETED_NO :
                return CompletionStatus.COMPLETED_NO; 

            case _COMPLETED_MAYBE :
                return CompletionStatus.COMPLETED_MAYBE; 
        }
    }

    /**
     * Returns a string representation.
     *
     * @return String description
     *
     * @since ACT 0.1
     */
	public String toString() {
        String types[] = {"COMPLETED_YES",
                          "COMPLETED_NO",
                          "COMPLETED_MAYBE",};
		return types[value];
	}
}
